package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.entity.KlConcept;
import com.diagbot.entity.KlDiagnose;
import com.diagbot.enums.DiagnoseLexiconTypeEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.LexiconExtEnum;
import com.diagbot.service.KlConceptService;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RegexUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.VerifyUtil;
import com.diagbot.vo.DiagnoseImportVO;
import com.diagbot.vo.ImportDiagnoseResVO;
import com.diagbot.vo.ImportDiagnoseVO;
import com.diagbot.vo.KlDiagnoseDetailVO;
import com.diagbot.vo.KlDiagnoseGroupVO;
import com.diagbot.vo.KlDiagnoseSaveVO;
import com.diagbot.vo.KlDiagnoseTypeVO;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.openxml4j.util.ZipSecureFile;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @Description:
 * @author: zhoutg
 * @time: 2021/3/15 11:42
 */
@Component
public class KlDiagnoseImportFacade {

    @Autowired
    @Qualifier("klConceptServiceImpl")
    private KlConceptService klConceptService;
    @Autowired
    KlDiagnoseFacade klDiagnoseFacade;

    public Map<String, Object> importDiagnose(MultipartFile file, DiagnoseImportVO diagnoseImportVO) {
        long t1 = System.currentTimeMillis();
        Map<String, Object> message = new LinkedHashMap<>(); // 提示信息
        List<ImportDiagnoseResVO> allData = Lists.newArrayList();  // 最终需要保存的数据
        if (StringUtil.isNotBlank(diagnoseImportVO.getDiagStr())) {
            List<String> diagList = Arrays.stream(diagnoseImportVO.getDiagStr().split("#")).map(r -> r.trim()).collect(Collectors.toList());
            diagnoseImportVO.setDiags(diagList);
        }
        // 校验数据
        Boolean allFlag = verifyData(file, diagnoseImportVO, message, allData);

        // 保存数据
        if (allFlag) {
            saveData(allData);
            message.put("导入条数", allData.size() + "条");
            // 如果有指定诊断，提示导入的诊断；如果全部导入，提示“所有诊断”
            if (ListUtil.isNotEmpty(diagnoseImportVO.getDiags())) {
                message.put("导入诊断", allData.stream().map(r -> r.getDiseaseName()).collect(Collectors.joining("#")));
            } else {
                message.put("导入诊断", "所有诊断");
            }
        }
        long t2 = System.currentTimeMillis();
        message.put("总计耗时", (t2 - t1) / 1000.0);
        return message;
    }

    /**
     * 校验数据
     *
     * @param file
     * @param diagnoseImportVO
     * @param diagMessage
     * @param allData
     * @return
     */
    public Boolean verifyData(MultipartFile file, DiagnoseImportVO diagnoseImportVO, Map<String, Object> diagMessage, List<ImportDiagnoseResVO> allData) {
        Boolean allFlag = true;
        List<String> diagtypes = Lists.newArrayList("拟诊", "确诊", "警惕");
        // 获取标准词库Map
        Map<String, KlConcept> concept_map = getConceptMap();

        // 分组之后的数据
        Map<String, List<ImportDiagnoseVO>> groupData = new LinkedHashMap<>();
        // 获取全部数据，分组
        List<ImportDiagnoseVO> data = ExcelUtils.importExcelMultiSheets(file, 0, 1, 0, ImportDiagnoseVO.class);
        List<ImportDiagnoseVO> importDiagnoseVOList = null;
        String addDisName = "";
        for (ImportDiagnoseVO importDiagnoseVO : data) {
            String type = importDiagnoseVO.getType();
            if (type != null && type.startsWith("【") && type.endsWith("】")) {
                if (StringUtil.isNotBlank(addDisName)) {
                    addDisName = RegexUtil.getRegexData(addDisName, "【(.*?)】", 1);
                    groupData.put(addDisName, importDiagnoseVOList);
                }
                importDiagnoseVOList = new ArrayList<>();
                addDisName = type;
            } else {
                importDiagnoseVOList.add(importDiagnoseVO);
            }
        }
        addDisName = RegexUtil.getRegexData(addDisName, "【(.*?)】", 1); // 获取名字
        groupData.put(addDisName, importDiagnoseVOList); // 添加最后一个

        int i = 0;
        for (String disName : groupData.keySet()) {
            // 指定诊断导入
            if (ListUtil.isNotEmpty(diagnoseImportVO.getDiags())) {
                if (!diagnoseImportVO.getDiags().contains(disName)) {
                    continue;
                }
            }
            // System.out.println(i++ + ": " + disName);
            List<String> currentErrMsg = Lists.newArrayList();

            ImportDiagnoseResVO importDiagnoseResVO = new ImportDiagnoseResVO();
            // 校验疾病名称【sheetName】
            KlConcept klConcept = concept_map.get(disName + "_" + String.valueOf(LexiconEnum.Disease.getKey()));
            if (klConcept != null) {
                importDiagnoseResVO.setDiseaseName(disName);
                importDiagnoseResVO.setDiseaseId(klConcept.getId());
            } else {
                currentErrMsg.add(String.format("%s不是标准词", disName));
            }
            // 校验其他行数据、格式
            List<ImportDiagnoseVO> curData = groupData.get(disName);

            //处理公式（确诊）
            List<String> orderList = processFormula(curData, diagtypes);
            dealVerify(currentErrMsg, orderList, diagtypes, curData, concept_map);
            // 数据赋值
            if (ListUtil.isEmpty(currentErrMsg)) {
                curData = dealImportDiagnose(diagtypes, curData, concept_map);
                // 添加到正确数据列表
                importDiagnoseResVO.setImportDiagnoseVOList(curData);
                allData.add(importDiagnoseResVO);
            } else {
                allFlag = false;
                diagMessage.put(disName, StringUtils.join(currentErrMsg.stream().distinct().collect(Collectors.toList()), "；  "));
            }
        }
        return allFlag;
    }

    private List<String> processFormula(List<ImportDiagnoseVO> data, List<String> diagtypes) {
        List<String> orderList = Lists.newArrayList();
        Map<String, List<String>> typeCollect = Maps.newHashMap();
        if (ListUtil.isNotEmpty(data)) {
            ListIterator<ImportDiagnoseVO> it = data.listIterator();
            while (it.hasNext()) {
                ImportDiagnoseVO next = it.next();
                String type = next.getType();
                String orderNo = next.getOrderNo();
                String rule = next.getRule();
                Integer rowNum = next.getRowNum() - 1;
                if (!diagtypes.contains(type) && StringUtil.isNotBlank(type) && StringUtil.isNotBlank(orderNo)) {
                    orderList.add(orderNo);
                }
                if ("拟诊".equals(type) && StringUtils.isNotBlank(rule)) {
                    if (typeCollect.containsKey(type)) {
                        List<String> gongsi = typeCollect.get(type);
                        gongsi.add(rule);
                        typeCollect.put(type, gongsi);
                    } else {
                        List<String> gongsi = Lists.newArrayList();
                        gongsi.add(rule);
                        typeCollect.put(type, gongsi);
                    }
                }
                if ("确诊".equals(type) && StringUtils.isNotBlank(rule)) {
                    if (rule.contains("拟诊") && typeCollect.containsKey("拟诊")) {
                        it.remove();
                        for (String nizhen : typeCollect.get("拟诊")) {
                            ImportDiagnoseVO importDiagnoseVO = new ImportDiagnoseVO();
                            importDiagnoseVO.setParentRule(next.getRule());
                            importDiagnoseVO.setType(type);
                            importDiagnoseVO.setRowNum(rowNum);
                            importDiagnoseVO.setOrderNo(rule.replace("拟诊", nizhen));
                            it.add(importDiagnoseVO);
                        }
                    }
                }
            }
        }
        return orderList;
    }

    /**
     * 获取sheet名称和序号的对应关系
     *
     * @param file
     * @return
     */
    private Map<String, Integer> dealExcel(MultipartFile file) {
        Map<String, Integer> sheetNumName = Maps.newLinkedHashMap();
        Workbook workBook = null;
        try {
            ZipSecureFile.setMinInflateRatio(-1.0d);
            workBook = ExcelUtils.getWorkBook(file);
            int numberOfSheets = workBook.getNumberOfSheets();
            for (int i = 0; i < numberOfSheets; i++) {
                String sheetName = workBook.getSheetName(i);
                if (!"目录".equals(sheetName)) {
                    sheetNumName.put(sheetName, i);
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return sheetNumName;
    }

    /**
     * 给标准词设置conceptId
     *
     * @param diagtypes
     * @param importDiagnoseVOS
     * @param concept_map
     * @return
     */
    private List<ImportDiagnoseVO> dealImportDiagnose(List<String> diagtypes, List<ImportDiagnoseVO> importDiagnoseVOS, Map<String, KlConcept> concept_map) {
        for (ImportDiagnoseVO idv : importDiagnoseVOS) {
            String type = idv.getType();
            String orderNo = idv.getOrderNo();
            String rule = idv.getRule();
            if (StringUtil.isBlank(type) && StringUtil.isBlank(orderNo)) {
                continue;
            }
            if (!type.contains("正则") && !diagtypes.contains(type)) {
                KlConcept klConcept = concept_map.get(rule + "_" + String.valueOf(LexiconExtEnum.getEnum(type).getKey()));
                if (klConcept != null) {
                    idv.setConceptId(klConcept.getId());
                }
            }
        }
        return importDiagnoseVOS;
    }

    //数据及格式校验
    private void dealVerify(List<String> currentErrMsg, List<String> orderNums, List<String> diagtypes, List<ImportDiagnoseVO> data, Map<String, KlConcept> concept_map) {
        //list元素计数
        Map<String, Long> orderNumMap = orderNums.stream().collect(Collectors.groupingBy(Function.identity(), Collectors.counting()));
        for (Map.Entry<String, Long> on : orderNumMap.entrySet()) {
            if (on.getValue() > 1) {
                currentErrMsg.add(String.format("%s序号重复", "【" + on.getKey() + "】"));
            }
        }

        if (ListUtil.isNotEmpty(data)) {
            for (ImportDiagnoseVO imd : data) {
                Integer rowNum = imd.getRowNum();
                String type = imd.getType();
                String rule = imd.getRule();
                String orderNo = imd.getOrderNo();
                String eq = imd.getEq();
                String maxSymbol = imd.getMaxSymbol();
                String minSymbol = imd.getMinSymbol();
                String parentRule = imd.getParentRule();
                if (StringUtil.isBlank(type) && StringUtil.isBlank(orderNo)) {
                    continue;
                }

                if (StringUtil.isNotBlank(type) && StringUtil.isNotBlank(rule) && StringUtil.isNotBlank(orderNo)) {
                    LexiconExtEnum anEnum = LexiconExtEnum.getEnum(type);
                    if (anEnum == null) {
                        currentErrMsg.add(String.format("第%d行【%s】类型错误", rowNum, type));
                        continue;
                    }
                    int key = anEnum.getKey();
                    if (!type.contains("正则") &&
                            !concept_map.containsKey(rule + "_" + String.valueOf(key))) {
                        currentErrMsg.add(String.format("第%d行【%s】不是标准词", rowNum, rule));
                    }
                    String name = DiagnoseLexiconTypeEnum.getName(key);
                    Boolean regexRes = RegexUtil.getRegexRes(orderNo, name + "\\d+");
                    if (!regexRes) {
                        currentErrMsg.add(String.format("第%d行类型【%s】和序号【%s】对应错误", rowNum, type, orderNo));
                    }
                }
                if (StringUtil.isNotBlank(maxSymbol) &&
                        !"<=".equals(maxSymbol) && !"<".equals(maxSymbol)) {
                    currentErrMsg.add(String.format("第%d行%s的【最大值符号】不正确", rowNum, rule));
                }
                if (StringUtil.isNotBlank(minSymbol) &&
                        !">=".equals(minSymbol) && !">".equals(minSymbol)) {
                    currentErrMsg.add(String.format("第%d行%s的【最小值符号】不正确", rowNum, rule));
                }
                if (LexiconExtEnum.LisSubName.getName().equals(type) && StringUtils.isNotBlank(rule)
                        && StringUtils.isBlank(eq)) {
                    currentErrMsg.add(String.format("第%d行%s的【等于值】不能为空", rowNum, rule));
                }
                if ((LexiconExtEnum.Vital.getName().equals(type) || LexiconExtEnum.Age.getName().equals(type))
                        && StringUtils.isNotBlank(rule)
                        && StringUtils.isBlank(imd.getMin()) && StringUtils.isBlank(imd.getMax())) {
                    currentErrMsg.add(String.format("第%d行%s【最大值】和【最小值】至少有一个不能为空", rowNum, rule));
                }
                if (StringUtils.isBlank(type)) {
                    currentErrMsg.add(String.format("第%d行【类型】不能为空", rowNum));
                }
                if (StringUtils.isNotBlank(type) && diagtypes.contains(type) && StringUtils.isBlank(rule)
                && StringUtils.isBlank(orderNo)) {
                    currentErrMsg.add(String.format("第%d行【诊断依据规则】不能为空", rowNum));
                }
                if (StringUtils.isNotBlank(type) && !diagtypes.contains(type) && StringUtils.isBlank(rule)) {
                    currentErrMsg.add(String.format("第%d行【规则】不能为空", rowNum));
                }
                if (diagtypes.contains(type) && StringUtil.isNotBlank(rule)) {
                    String s = VerifyUtil.verifyFormula(orderNums, rule);
                    if (StringUtil.isNotBlank(s)) {
                        if (StringUtil.isNotBlank(parentRule)) {
                            s = s.replace(rule, parentRule);
                        }
                        currentErrMsg.add(s);
                    }
                }
            }


            //格式校验
            /*if (ListUtil.isNotEmpty(verifies)) {
                for (String vef : verifies) {
                    String s = VerifyUtil.verifyFormula(orderNums, vef);
                    if (StringUtil.isNotBlank(s)) {
                        currentErrMsg.add(s);
                    }
                }
            }*/
        }
    }

    private Map<String, KlConcept> getConceptMap() {
        List<KlConcept> klConcepts
                = klConceptService.list(new QueryWrapper<KlConcept>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
        );
        Map<String, KlConcept> map
                = EntityUtil.makeEntityMapByKeys(klConcepts, "_", "libName", "libType");
        return map;
    }

    /**
     * 保存数据
     *
     * @param allData
     */
    public void saveData(List<ImportDiagnoseResVO> allData) {
        for (ImportDiagnoseResVO importDiagnoseResVO : allData) {
            Long diseaseId = importDiagnoseResVO.getDiseaseId();
            String diseaseName = importDiagnoseResVO.getDiseaseName();
            List<ImportDiagnoseVO> data = importDiagnoseResVO.getImportDiagnoseVOList();
            // 获取序号对应的行数据
            Map<String, List<ImportDiagnoseVO>> map = EntityUtil.makeEntityListMap(data, "orderNo");

            if (ListUtil.isNotEmpty(data)) {
                // 第一层
                KlDiagnoseSaveVO klDiagnoseSaveVO = new KlDiagnoseSaveVO();
                klDiagnoseSaveVO.setConceptId(diseaseId);
                klDiagnoseSaveVO.setDescription(diseaseName);

                // 第二层
                List<KlDiagnoseTypeVO> klDiagnoseTypeVOList = Lists.newArrayList();

                for (ImportDiagnoseVO importDiagnoseVO : data) {
                    String type = importDiagnoseVO.getType();
                    String rule = importDiagnoseVO.getRule();
                    if (StringUtil.isNotBlank(type) && StringUtil.isNotBlank(rule)) {
                        if ("拟诊".equals(type) || "确诊".equals(type) || "警惕".equals(type)) {
                            String[] splitGroup = rule.split("\\+");

                            KlDiagnoseTypeVO klDiagnoseTypeVO = new KlDiagnoseTypeVO();
                            if ("拟诊".equals(type)) {
                                klDiagnoseTypeVO.setConditionType(2);
                            } else if ("确诊".equals(type)) {
                                klDiagnoseTypeVO.setConditionType(3);
                            } else if ("警惕".equals(type)) {
                                klDiagnoseTypeVO.setConditionType(4);
                            }

                            //  第三层
                            List<KlDiagnoseGroupVO> groupVO = Lists.newArrayList();
                            for (String group : splitGroup) {
                                KlDiagnoseGroupVO klDiagnoseGroupVO = generateDiagnoseGroup(group, map);
                                groupVO.add(klDiagnoseGroupVO);
                            }

                            klDiagnoseTypeVO.setGroupVO(groupVO);
                            klDiagnoseTypeVOList.add(klDiagnoseTypeVO);
                            klDiagnoseSaveVO.setKlDiagnoseTypeVO(klDiagnoseTypeVOList);
                        }
                    }
                }

                // 获取主表数据
                KlDiagnose klDiagnose = klDiagnoseFacade.getOne(new QueryWrapper<KlDiagnose>().eq("concept_id", diseaseId), false);
                if (klDiagnose != null) {
                    klDiagnoseSaveVO.setId(klDiagnose.getId());
                    klDiagnoseSaveVO.setModifier("0");
                }
                klDiagnoseFacade.saveDiagnoseAll(klDiagnoseSaveVO);
            }
        }
    }


    /**
     * 生成KlDiagnoseGroupVO， 每一组+分隔后的内容
     *
     * @param map
     * @param text
     * @return
     */
    public KlDiagnoseGroupVO generateDiagnoseGroup(String text, Map<String, List<ImportDiagnoseVO>> map) {
        KlDiagnoseGroupVO klDiagnoseGroupVO = new KlDiagnoseGroupVO();

        //  list[0]:序号，list[1]:任几
        List<String> list = Lists.newArrayList();
        if (text.contains("[") || text.contains("]")) {
            list.add(text.substring(text.indexOf("[") + 1, text.indexOf("]")));
            String numStr = text.substring(text.indexOf("任") + 1);
            String num = "1";
            switch (numStr) {
                case "一":
                    num = "1";
                    break;
                case "二":
                    num = "2";
                    break;
                case "三":
                    num = "3";
                    break;
                case "四":
                    num = "4";
                    break;
                case "五":
                    num = "5";
                    break;
                case "六":
                    num = "6";
                    break;
                case "七":
                    num = "7";
                    break;
                case "八":
                    num = "8";
                    break;
                case "九":
                    num = "9";
                    break;
                case "十":
                    num = "10";
                    break;
            }
            list.add(num);
        } else {
            list.add(text);
            list.add("1");
        }
        klDiagnoseGroupVO.setFitNo(Integer.valueOf(list.get(1))); // 任几

        List<KlDiagnoseDetailVO> klDiagnoseDetail = Lists.newArrayList();
        String[] orderArr = list.get(0).split("、");
        for (String order : orderArr) {
            KlDiagnoseDetailVO klDiagnoseDetailVO = new KlDiagnoseDetailVO();
            ImportDiagnoseVO current = map.get(order).get(0);
            LexiconExtEnum libTypeEnum = LexiconExtEnum.getEnum(current.getType());
            if (libTypeEnum != null) {
                klDiagnoseDetailVO.setBasLibName(current.getRule());
                switch (libTypeEnum) {
                    case Symptom: // 症状
                    case VitalResult: // 体格检查结果
                    case PacsResult: // 辅助检查结果
                    case Group: // 人群
                    case Disease: // 疾病
                        klDiagnoseDetailVO.setBasConceptId(current.getConceptId());
                        klDiagnoseDetailVO.setBasType(1);
                        klDiagnoseDetailVO.setBasDescription(current.getRule());
                        break;
                    case zsxbszz: // 主诉现病史正则
                        klDiagnoseDetailVO.setBasType(3);
                        klDiagnoseDetailVO.setBasDescription(current.getRule());
                        klDiagnoseDetailVO.setEqValue(current.getRule());
                        break;
                    case jwzz: // 既往史正则
                        klDiagnoseDetailVO.setBasType(4);
                        klDiagnoseDetailVO.setBasDescription(current.getRule());
                        klDiagnoseDetailVO.setEqValue(current.getRule());
                        break;
                    case ctzz: // 查体正则
                        klDiagnoseDetailVO.setBasType(5);
                        klDiagnoseDetailVO.setBasDescription(current.getRule());
                        klDiagnoseDetailVO.setEqValue(current.getRule());
                        break;
                    case fjjgzz: // 辅检结果正则
                        klDiagnoseDetailVO.setBasType(6);
                        klDiagnoseDetailVO.setBasDescription(current.getRule());
                        klDiagnoseDetailVO.setEqValue(current.getRule());
                        break;
                    case Vital: // 体格检查项目
                    case Age: // 年龄
                        klDiagnoseDetailVO.setBasConceptId(current.getConceptId());
                        if (StringUtil.isNotBlank(current.getMin())) {
                            klDiagnoseDetailVO.setMinVal(current.getMin());
                        }
                        if (StringUtil.isNotBlank(current.getMinSymbol())) {
                            klDiagnoseDetailVO.setMinOperator(current.getMinSymbol());
                        }
                        if (StringUtil.isNotBlank(current.getUnit())) {
                            klDiagnoseDetailVO.setMinUnit(current.getUnit());
                            klDiagnoseDetailVO.setMaxUnit(current.getUnit());
                        }
                        if (StringUtil.isNotBlank(current.getMax())) {
                            klDiagnoseDetailVO.setMaxVal(current.getMax());
                        }
                        if (StringUtil.isNotBlank(current.getMaxSymbol())) {
                            klDiagnoseDetailVO.setMaxOperator(String.valueOf(current.getMaxSymbol()));
                        }
                        klDiagnoseDetailVO.setBasDescription(current.getRule());
                        klDiagnoseDetailVO.setBasType(2);
                        break;
                    case LisSubName: // 实验室检查子项目
                        klDiagnoseDetailVO.setBasConceptId(current.getConceptId());
                        klDiagnoseDetailVO.setEqValue(current.getEq());
                        klDiagnoseDetailVO.setBasDescription(current.getRule() + current.getEq());
                        klDiagnoseDetailVO.setBasType(2);
                        break;
                }
            }
            klDiagnoseDetail.add(klDiagnoseDetailVO);
        }

        klDiagnoseGroupVO.setKlDiagnoseDetail(klDiagnoseDetail);
        return klDiagnoseGroupVO;
    }

    public static void main(String[] args) {
        String text = "[1、2、3]任一";
        List<String> list = Lists.newArrayList();
        if (text.contains("[") || text.contains("]")) {
            list.add(text.substring(text.indexOf("[") + 1, text.indexOf("]")));
            list.add(text.substring(text.indexOf("]") + 1));
        }
        System.out.println(list);
    }
}
